<?php
/**
 * class Frontend
 * @package DaReactions
 * @since 1.0.0
 */
namespace DaReactions;
/**
 * Manages all frontend tasks such as print reactions buttons and load scripts and styles
 *
 * class Frontend
 * @package DaReactions
 * @since 1.0.0
 */
class Frontend
{
    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;
    /**
     * Initialize the class and set its properties.
     *
     * @param Main
     *
     * @since    1.0.0
     *
     */
    public function __construct(Main $main)
    {
        $this->plugin_name = $main->getPluginName();
    }
    /**
     * Inject reaction buttons HTML in comment content
     *
     * @param $comment_text
     *
     * @return string
     *
     * @since 1.0.0
     */
    public function addButtonsToComment($comment_text)
    {
        global $post;
        $options = Options::getInstance('general');
        $globally_enabled = $options->getOption('post_type_' . $post->post_type . '_comments') === 'on';
        $locally_enabled = $options->getOption('post_type_' . $post->post_type . '_enable_comments_' . $post->ID) === 'on';
        $locally_disabled = $options->getOption('post_type_' . $post->post_type . '_disable_comments_' . $post->ID) === 'on';
        $enabled = ($globally_enabled && !$locally_disabled) || $locally_enabled;
        if (!$enabled) {
            return $comment_text;
        }
        global $comment;
        if (is_singular()) {
            $item_type = 'comment';
            $item_id = $comment->comment_ID;
            $append = self::getButtonsPlaceholder($item_type, $item_id);
            return $comment_text . $append;
        }
        return $comment_text;
    }
    /**
     * Inject reaction buttons HTML in post content
     *
     * @param $content
     *
     * @return string
     *
     * @since 1.0.0
     */
    public function addButtonsToContent($content)
    {
        global $post;
        if (!$post) {
            return false;
        }
        $general_options = Options::getInstance('general');
        $globally_enabled = $general_options->getOption('post_type_' . $post->post_type) === 'on';
        $locally_enabled = $general_options->getOption('post_type_' . $post->post_type . '_enable_' . $post->ID) === 'on';
        $locally_disabled = $general_options->getOption('post_type_' . $post->post_type . '_disable_' . $post->ID) === 'on';
        $enabled = ($globally_enabled && !$locally_disabled) || $locally_enabled;
        if ($enabled) {
            if (is_home()) {
                // Default homepage
                $enabled = $general_options->getOption('page_type_blog') === 'on';
            } else if (is_archive()) {
                // archive page
                $enabled = $general_options->getOption('page_type_archive') === 'on';
            } else if (is_singular()) {
                $enabled = $general_options->getOption('page_type_single') === 'on';
            }
        }
        if (!$enabled) {
            return $content;
        }
        $item_type = $post->post_type;
        $item_id = $post->ID;
        $append = self::getButtonsPlaceholder($item_type, $item_id);
        return $content . $append;
    }
    /**
     * Inject reaction buttons HTML in post excerpt
     *
     * @param $excerpt
     *
     * @return string
     *
     * @since 1.0.0
     */
    public function addButtonsToExcerpt($excerpt)
    {
        return $this->addButtonsToContent($excerpt);
    }
    /**
     * Registers the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueueStyles()
    {
        $graphic_options = Options::getInstance('graphic');
        wp_enqueue_style(
            $this->plugin_name,
            DA_REACTIONS_URL . 'assets/dist/public.css',
            array(),
            DA_REACTIONS_VERSION,
            'all'
        );
        $fade_amount = absint($graphic_options->getOption('fade_value', 50));
        $fade_method = $graphic_options->getOption('fade_method', 'none');
        $size = absint($graphic_options->getOption('button_size', 64));
        if (wp_is_mobile() && $graphic_options->getOption('da_r_mobile_enabled', 'off') === 'on') {
            $fade_amount = absint($graphic_options->getOption('fade_value_mobile', 50));
            $fade_method = $graphic_options->getOption('fade_method_mobile', 'none');
            $size = absint($graphic_options->getOption('button_size_mobile', 64));
        }
        $inline_css = self::getInlineCss($fade_method, $fade_amount, $size);
        wp_add_inline_style($this->plugin_name, $inline_css);
    }
    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueueScripts()
    {
        $general_options = Options::getInstance('general');
        $graphic_options = Options::getInstance('graphic');
        $show_count = $graphic_options->getOption('show_count', 'always');
        if (wp_is_mobile() && $graphic_options->getOption('da_r_mobile_enabled', 'off') === 'on') {
            $show_count = $graphic_options->getOption('show_count_mobile', 'always');
        }
        wp_enqueue_script(
            $this->plugin_name,
            DA_REACTIONS_URL . 'assets/dist/public.js',
            array(
                'jquery',
            ),
            DA_REACTIONS_VERSION,
            false
        );
        wp_localize_script($this->plugin_name, 'DaReactions', array(
            'ajax_url' => admin_url('admin-ajax.php'),
            'display_detail_modal' => $general_options->getOption('display_detail_modal', 'off'),
            'display_detail_modal_toolbar' => $general_options->getOption('display_detail_modal_toolbar', 'off'),
            'display_detail_tooltip' => $general_options->getOption('display_detail_tooltip', 'off'),
            'loader_url' => DA_REACTIONS_URL . 'assets/dist/loading.svg',
            'modal_result_limit' => absint($general_options->getOption('modal_result_limit', 100)),
            'nonce' => wp_create_nonce('nonce'),
            'show_count' => $show_count,
            'tooltip_result_limit' => absint($general_options->getOption('tooltip_result_limit', 5)),
            'labels' => array(
                'modal_tab_all_title' => __('All reactions', 'da-reactions'),
                'modal_no_tabs_title' => __('Reactions', 'da-reactions'),
                'modal_pagination_close' => __('×', 'da-reactions'),
                'modal_pagination_next' => __('→', 'da-reactions'),
                'modal_pagination_prev' => __('←', 'da-reactions'),
                'modal_pagination_desc' => __('Page {current} of {total}', 'da-reactions'),
            )
        ));
    }
    /**
     * Generates a placeholder to load buttons asyncronously
     *
     * @param $item_type
     * @param $item_id
     *
     * @return string
     *
     * @since 1.0.0
     */
    public static function getButtonsPlaceholder($item_type, $item_id)
    {
        $graphic_options = Options::getInstance('graphic');
        $size = absint($graphic_options->getOption('button_size', 64));
        $alignment = $graphic_options->getOption('buttons_alignment', 'center');
        $before_reactions = wpautop($graphic_options->getOption('description_text', ''));
        $template = $graphic_options->getOption('use_template', 'exposed');
        $element_id = 'da-reactions-slot-' . $item_type . '-' . absint($item_id);
        $item_id_value = absint($item_id);
        $nonce = wp_create_nonce($item_type . '-' . $item_id_value);
        $single_reaction_image = self::getSingleReactionImage(
            DA_REACTIONS_URL . 'assets/dist/loading.svg',
            $size,
            'Loading spinner'
        );
        return "
		<div class=\"da-reactions-outer T${item_type}ID${item_id_value}\">
            $before_reactions
            <div class=\"da-reactions-data da-reactions-container-async ${alignment}\"
                data-type=\"${item_type}\"
                data-id=\"${item_id_value}\"
                data-nonce=\"${nonce}\"
                id=\"${element_id}\">
                <div class=\"da-reactions-${template}\">
                    ${single_reaction_image}
                </div>
            </div>
        </div>";
    }
    /**
     * Return html string for count badge
     *
     * @param $count
     * @param $show_count
     *
     * @return string
     */
    public static function getCountBadge($count, $percentage, $show_count = '')
    {
        $graphic_options = Options::getInstance('graphic');
        if ($show_count === '') {
            $show_count = $graphic_options->getOption('show_count', 'always');
            if (wp_is_mobile() && $graphic_options->getOption('da_r_mobile_enabled', 'off') === 'on') {
                $show_count = $graphic_options->getOption('show_count_mobile', 'always');
            }
        }
        $style = '';
        switch ($show_count) {
            case 'non-zero':
            case 'percentage-non-zero':
                if ($count <= 0) {
                    $style = 'style="display: none;"';
                }
                break;
            case 'never':
                $style = 'style="display: none;"';
                break;
        }
        if (strpos($show_count, 'percentage') === 0) {
            $number_value = intval($percentage) . '%';
        } else {
            $number_value = Utils::formatBigNumber($count);
        }
        return '<div class="count" ' . $style . ' data-value="' . $number_value . '"></div>';
    }
    /**
     * Generates HTML for buttons
     * Used in renderTemplateExposed and in renderTemplateReveal
     *
     * @param $reactions
     * @param $item_id
     * @param $nonce
     * @param $item_type
     * @param $size
     * @param $show_count
     *
     * @return string
     */
    public static function getImageButtonsHtml($reactions, $item_id, $nonce, $item_type, $size, $show_count = '')
    {
        $op = '';
        foreach ($reactions as $reaction) {
            $count = $reaction->total;
            $percentage = $reaction->percentage;
            $op .= sprintf('
				<div class="da-reactions-data reaction reaction_%1$d %2$s" data-id="%3$d" data-nonce="%4$s" data-reaction="%1$d" data-title="%5$s" data-type="%6$s">
					%7$s
					%8$s
				</div>
				',
                $reaction->ID,
                (@$reaction->current ? 'active' : 'inactive'),
                $item_id,
                $nonce,
                $reaction->label,
                $item_type,
                self::getSingleReactionImage(FileSystem::getImageUrl($reaction->file_name), $size, $reaction->label),
                self::getCountBadge($count, $percentage, $show_count)
            );
        }
        return $op;
    }
    /**
     * Return css string for inline style
     *
     * @param $fade_method
     * @param $fade_amount
     * @param $size
     *
     * @return string
     */
    public static function getInlineCss($fade_method, $fade_amount, $size)
    {
        $inline_css = '';
        switch ($fade_method) {
            case 'transparence':
                $value = 1 - ($fade_amount / 100);
                $inline_css = "
		            div.da-reactions-container.has_current div.reaction img {
                        opacity: $value;
                    }
                    div.da-reactions-container.has_current div.reaction:hover img,
                    div.da-reactions-container.has_current div.reaction.active img {
                        opacity: 1;
                    }";
                break;
            case 'desaturate':
                $value = $fade_amount;
                $inline_css = "
		            div.da-reactions-container.has_current div.reaction img {
                        filter: grayscale($value%);
                    }
                    div.da-reactions-container.has_current div.reaction:hover img,
                    div.da-reactions-container.has_current div.reaction.active img {
                        filter: none;
                    }";
                break;
            case 'blur':
                $value = $fade_amount * $size / 200;
                $inline_css = "
		            div.da-reactions-container.has_current div.reaction img {
                        filter: blur(${value}px);
                    }
                    div.da-reactions-container.has_current div.reaction:hover img,
                    div.da-reactions-container.has_current div.reaction.active img {
                        filter: none;
                    }";
                break;
        }
        return $inline_css;
    }
    /**
     * Generates Reaction buttons HTML markup
     *
     * @param $item_type
     * @param $item_id
     *
     * @return string
     *
     * @since 1.0.0
     */
    public static function getButtons($item_type, $item_id)
    {
        $graphic_options = Options::getInstance('graphic');
        $template = $graphic_options->getOption('use_template', 'exposed');
        if (wp_is_mobile() && $graphic_options->getOption('da_r_mobile_enabled', 'off') === 'on') {
            $template = $graphic_options->getOption('use_template_mobile', 'exposed');
        }
        $reactions = Data::getReactionsForContent($item_id, $item_type);
        return self::renderTemplate($template, $reactions, $graphic_options, $item_type, $item_id);
    }
    /**
     * Generates an hamburger menu toggle for mobile
     *
     * @param int $size The size in pixels
     *
     * @return string
     *
     * @since 1.0.0
     */
    private static function getToggleButton($size = 64)
    {
        return sprintf('
			<div class="reactions-toggle">
				<svg xmlns="http://www.w3.org/2000/svg" x="0px" y="0px" viewBox="0 0 100 100" width="%1$d" height="%1$d" xml:space="preserve">
					<path class="bar1" d="M100,10.716c0,4.261-3.455,7.716-7.716,7.716H7.717c-4.261,0-7.716-3.455-7.716-7.716l0,0C0.001,6.455,3.455,3,7.717,3h84.567C96.545,3,100,6.455,100,10.716L100,10.716z"/>
					<path class="bar2" d="M100,49.732c0,4.262-3.455,7.716-7.716,7.716H7.717c-4.261,0-7.716-3.454-7.716-7.716l0,0c0-4.261,3.455-7.716,7.716-7.716h84.567C96.545,42.016,100,45.471,100,49.732L100,49.732z"/>
					<path class="bar3" d="M100,88.749c0,4.261-3.455,7.716-7.716,7.716H7.717c-4.261,0-7.716-3.455-7.716-7.716l0,0c0-4.261,3.455-7.716,7.716-7.716h84.567C96.545,81.033,100,84.487,100,88.749L100,88.749z"/>
				</svg>
			</div>',
            $size
        );
    }
    /**
     * Generates HTML markup for template if exists
     *
     * @param $templateName
     * @param $reactions
     * @param $options
     * @param $item_type
     * @param $item_id
     * @param $show_count
     *
     * @return string
     */
    public static function renderTemplate($templateName, $reactions, $options, $item_type, $item_id, $show_count = '')
    {
        switch ($templateName) {
            case 'exposed':
                return self::renderTemplateExposed($reactions, $options, $item_type, $item_id, $show_count);
            case 'reveal':
                return self::renderTemplateReveal($reactions, $options, $item_type, $item_id, $show_count);
            case 'static':
                return self::renderTemplateStatic($reactions, $options, $item_type, $item_id, $show_count);
            default:
                return 'Template not found error [' . $templateName . ']';
        }
    }
    /**
     *
     * Generate HTML markup for the “exposed” template
     *
     * @param $reactions
     * @param $graphic_options
     * @param $item_type
     * @param $item_id
     * @param $show_count
     *
     * @return string
     */
    public static function renderTemplateExposed($reactions, $graphic_options, $item_type, $item_id, $show_count = '')
    {
        $settings = Data::getReactionsSettings($reactions, $graphic_options, $item_type, $item_id);
        $alignment = $settings['alignment'];
        $has_current = $settings['has_current'] ? 'has_current' : '';
        $toggle_button = self::getToggleButton($settings['size']);
        $button_html = self::getImageButtonsHtml($reactions, $item_id, $settings['nonce'], $item_type, $settings['size'], $show_count);
        return sprintf('
			<div class="da-reactions-container %s %s">
				%s
				<div class="reactions responsive">
					%s
				</div>
			</div>
		',
            $alignment,
            $has_current,
            $toggle_button,
            $button_html
        );
    }
    /**
     * Generate HTML markup for the “reveal” template
     *
     * @param $reactions
     * @param $graphic_options
     * @param $item_type
     * @param $item_id
     * @param $show_count
     *
     * @return string
     */
    public static function renderTemplateReveal($reactions, $graphic_options, $item_type, $item_id, $show_count = '')
    {
        $settings = Data::getReactionsSettings($reactions, $graphic_options, $item_type, $item_id);
        $show_count_before_reveal = array(
            'always' => 'always',
            'percentage' => 'always',
            'non-zero' => 'non-zero',
            'percentage-non-zero' => 'non-zero',
            'never' => 'never'
        );
        return sprintf('
			<div class="da-reactions-reveal %s">
				<div class="before-reveal">
					%s
					%s
				</div>
				<div class="after-reveal da-reactions-container %s">
					%s
				</div>
			</div>
		',
            $settings['alignment'],
            self::getSingleReactionImage($settings['visible_reaction_image'], $settings['size']),
            self::getCountBadge($settings['total_count'], '100', $show_count_before_reveal[$show_count]),
            ($settings['has_current'] ? 'has_current' : ''),
            self::getImageButtonsHtml($reactions, $item_id, $settings['nonce'], $item_type, $settings['size'], $show_count)
        );
    }
    /**
     *
     * Generate HTML markup for the “exposed” template
     *
     * @param $reactions
     * @param $graphic_options
     * @param $item_type
     * @param $item_id
     * @param $show_count
     *
     * @return string
     */
    public static function renderTemplateStatic($reactions, $graphic_options, $item_type, $item_id, $show_count = '')
    {
        $settings = Data::getReactionsSettings($reactions, $graphic_options, $item_type, $item_id);
        return sprintf('
			<div class="da-reactions-container %s %s">
				<div class="reactions">
					%s
				</div>
			</div>',
            $settings['alignment'],
            ($settings['has_current'] ? 'has_current' : ''),
            self::getImageButtonsHtml($reactions, $item_id, $settings['nonce'], $item_type, $settings['size'], $show_count)
        );
    }
    /**
     * @param string $src
     * @param int $size
     * @param string $alt
     *
     * @return string
     */
    public static function getSingleReactionImage($src, $size = 64, $alt = 'Add your reaction')
    {
        return sprintf(/** @lang php */ '<img src="%1$s" alt="%2$s" width="%3$d" height="%3$d" style="width:%3$dpx; height:%3$dpx" />',
            $src,
            __($alt, 'da-reactions'),
            $size);
    }
}
