<?php
/**
 * Class GeneralSettings
 * @package DaReactions\Pages
 *
 * Admin general settings page
 *
 * @since 1.0.0
 */
namespace DaReactions\Pages;
use DaReactions\Data;
use DaReactions\Main;
use DaReactions\Options;
use DaReactions\Plugins\BuddyPress;
use DaReactions\User;
/**
 * Class GeneralSettings
 * @package DaReactions\Pages
 *
 * Admin general settings page
 *
 * @since 1.0.0
 */
class GeneralSettings extends SettingsPage
{
    /**
     * @var Main $main
     * A reference to Main instance
     *
     * @since 2.0.3
     */
    private $main;
    /**
     * GeneralSettings constructor.
     *
     * @param string $options_group
     * @param $section
     * @param Main $main
     *
     * @since 1.0.0
     */
    public function __construct($options_group, $section, Main $main)
    {
        parent::__construct($options_group, $section);
        $this->main = $main;
        $this->navigation = array();
        $this->navigation['general'] = array(
            'title' => __('General', 'da-reactions'),
            'visible' => true
        );
        $this->navigation['user'] = array(
            'title' => __('User', 'da-reactions'),
            'visible' => true
        );
            $this->navigation['detail'] = array(
                'title' => __('Detail', 'da-reactions'),
                'visible' => true
            );
        $this->navigation['performance'] = array(
            'title' => __('Performance', 'da-reactions'),
            'visible' => true
        );
        $this->navigation['plugins'] = array(
            'title' => __('Plugins', 'da-reactions'),
            'visible' => true
        );
        $this->navigation['preferences'] = array(
            'title' => __('Preferences', 'da-reactions'),
            'visible' => true
        );
        if (isset($_GET['tab'])) {
            $this->current_tab = filter_var($_GET['tab'], FILTER_SANITIZE_STRING);
        }
        if (
            !(
                array_key_exists($this->current_tab, $this->navigation)
                &&
                $this->navigation[$this->current_tab]['visible'] === true
            )
        ) {
            $this->current_tab = 'general';
        }
    }
    /**
     * Getter for this instace options
     *
     * @return Options
     */
    public function getOptions()
    {
        return $this->options;
    }
    /**
     * Register all settings for this page
     *
     * @since 1.0.0
     */
    public function initSettings()
    {
//        Data::createDummyVotes();
        register_setting(
            $this->options_group,
            $this->options_group,
            array($this, 'sanitizeData')
        );
        $valid_tab = false;
        if ($this->current_tab === 'general') {
            $this->registerGeneralSettings();
            $valid_tab = true;
        } else if ($this->current_tab === 'user') {
            $this->registerUserSettings();
            $valid_tab = true;
        } else if ($this->current_tab === 'plugins') {
            $this->registerPluginsSettings();
            $valid_tab = true;
        } else if ($this->current_tab === 'performance') {
            $this->registerPerformanceSettings();
            $valid_tab = true;
        } else if ($this->current_tab === 'preferences') {
            $this->registerPreferencesSettings();
            $valid_tab = true;
        }
            if ($this->current_tab === 'detail') {
                $this->registerDetailSettings__premium_only();
                $valid_tab = true;
            }
        if (!$valid_tab) {
            wp_die('May I help you?');
        }
    }
    /**
     * Register Buddypress Settings Section
     *
     * @since 3.1.1
     */
    public function registerPluginsSettings()
    {
        $count_plugins = 0;
        $title = _x('Third Party Plugins', 'Section title', 'da-reactions');
        // BBPress
        if ($this->main->isBBPressInstalled()) {
            $section = 'bbpress_section';
            $count_plugins++;
            add_settings_section(
                $section,
                _x('BBPress Options', 'Settings section', 'da-reactions'),
                $this->makeSectionRenderer(
                    __('These are BBPress specific integration settings.', 'da-reactions')
                ),
                $this->options_page
            );
            add_settings_field(
                'da_r_bbp_forum',
                __('Forums', 'da-reactions'),
                $this->makeCheckboxRenderer(
                    'bbp_forum_enabled',
                    __('Add reactions to BBPress’ forums.', 'da-reactions')
                ),
                $this->options_page,
                $section
            );
            add_settings_field(
                'da_r_bbp_topic',
                __('Topics', 'da-reactions'),
                $this->makeCheckboxRenderer(
                    'bbp_topic_enabled',
                    __('Add reactions to BBPress’ topics.', 'da-reactions')
                ),
                $this->options_page,
                $section
            );
            add_settings_field(
                'da_r_bbp_reply',
                __('Replies', 'da-reactions'),
                $this->makeCheckboxRenderer(
                    'bbp_reply_enabled',
                    __('Add reactions to BBPress’ replies.', 'da-reactions')
                ),
                $this->options_page,
                $section
            );
        }
        // BuddyPress
        if ($this->main->isBuddyPressInstalled__premium_only()) {
            $section = 'buddypress_section';
            $count_plugins++;
            $bp_manager = new BuddyPress();
            add_settings_section(
                $section,
                _x('BuddyPress Options', 'Settings section', 'da-reactions'),
                $this->makeSectionRenderer(
                    __('These are BuddyPress specific integration settings.', 'da-reactions')
                ),
                $this->options_page
            );
            add_settings_field(
                'da_r_bp_profile',
                __('Profiles', 'da-reactions'),
                $this->makeCheckboxRenderer(
                    'bp_profile_enabled',
                    __('Add reactions to BuddyPress’ profiles.', 'da-reactions')
                ),
                $this->options_page,
                $section
            );
            add_settings_field(
                'da_r_bp_activity',
                __('Activities', 'da-reactions'),
                array($bp_manager, 'renderActivityEnableCheckbox'),
                $this->options_page,
                $section
            );
            add_settings_field(
                'da_r_bp_group',
                __('Groups', 'da-reactions'),
                array($bp_manager, 'renderGroupEnableCheckbox'),
                $this->options_page,
                $section
            );
        }
        if ($count_plugins === 0) {
            $section = 'plugins_section';
            add_settings_section(
                $section,
                $title,
                $this->makeSectionRenderer(__('No supported third party plugins found.', 'da-reactions')),
                $this->options_page
            );
        }
    }
    /**
     * Register Detail Settings Section
     *
     * @since 3.1.1
     */
    public function registerDetailSettings__premium_only()
    {
        $section = 'details_section';
        $title = _x('Details', 'Section title', 'da-reactions');
        $intro = __('Enable those options if you want to display details about user who added reactions', 'da-reactions');
        add_settings_section(
            $section,
            $title,
            $this->makeSectionRenderer($intro),
            $this->options_page
        );
        add_settings_field(
            'da_r_display_users_tooltip',
            __('Show a tooltip with most recent reactions and users when mouse move over the count badge?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'display_detail_tooltip',
                __('Show detail tooltip on mouseover count badge . ', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_display_users_tooltip_limit',
            __('How many results should be shown on tooltip?', 'da-reactions'),
            $this->makeTextfieldRenderer(
                'tooltip_result_limit',
                '5',
                array(
                    'type' => 'number',
                    'min' => '1'
                )
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_display_users_modal',
            __('Show a modal with most recent reactions and link to user’s page/profile when mouse move over the count badge?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'display_detail_modal',
                __('Show detail modal on click on count badge . ', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_display_users_modal_limit',
            __('How many results should be shown on modal?', 'da-reactions'),
            $this->makeTextfieldRenderer(
                'modal_result_limit',
                '100',
                array(
                    'type' => 'number',
                    'min' => '1'
                )
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_display_users_modal_radio',
            __('Show toolbar on modal to filter results by reaction?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'display_detail_modal_toolbar',
                __('Show reaction tabs on modal.', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_display_anon_voters',
            __('Display anonymous voters?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'display_anonymous_voters',
                __('Show anonymous voters . ', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_anon_voters_name',
            __('Anonymous voters default name', 'da-reactions'),
            $this->makeTextfieldRenderer(
                'default_votername',
                'Unknown'
            ),
            $this->options_page,
            $section
        );
    }
    /**
     * Register General Settings Section
     *
     * @since 3.1.1
     */
    public function registerGeneralSettings()
    {
        $section = 'general_section';
        $title = __('General settings', 'da-reactions');
        $intro = __('These are general settings, you can select to enable or disable reactions for specific content types and views.', 'da-reactions');
        add_settings_section(
            $section,
            $title,
            $this->makeSectionRenderer($intro),
            $this->options_page
        );
        add_settings_field(
            'da_r_post_type_selector',
            __('Add reactions to post types and comments', 'da-reactions'),
            array($this, 'renderPostTypeSelector'),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_template_type_selector',
            __('Add reactions to single pages and/or archives too', 'da-reactions'),
            array($this, 'renderPageTypeSelector'),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_chart_colors_selector',
            __('Use different colors for chart widgets?', 'da-reactions'),
            array($this, 'renderChartColorSelector'),
            $this->options_page,
            $section
        );
    }
    /**
     * Register Performance Settings Section
     *
     * @since 3.1.1
     */
    public function registerPerformanceSettings()
    {
        $section = 'performance_section';
        $title = __('Performance settings', 'da-reactions');
        $intro = __('Use those settings to solve performances issues', 'da-reactions');
        add_settings_section(
            $section,
            $title,
            $this->makeSectionRenderer($intro),
            $this->options_page
        );
        add_settings_field(
            'da_r_use_cache',
            __('Use plugin own cache system to serve generated widgets from disk?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'enable_internal_cache',
                __('Enable internal plugin cache', 'da-reactions')
            ),
            $this->options_page,
            $section,
            array(
                'class' => 'enable_cache_selector'
            )
        );
    }
    /**
     * Register Plugin Settings Section
     *
     * @since 3.1.1
     */
    public function registerPreferencesSettings()
    {
        $section = 'plugin_section';
        $title = __('Plugin settings', 'da-reactions');
        $intro = __('These are plugin settings, select a method to identify unlogged users and check the delete option if you want to get rid of all data on plugin deactivation . ', 'da-reactions');
        add_settings_section(
            $section,
            $title,
            $this->makeSectionRenderer($intro),
            $this->options_page
        );
        add_settings_field(
            'da_r_identification_method',
            __('Which identification method should be used for unregistered users?', 'da-reactions'),
            array($this, 'renderUserIdentificationMethod'),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_remove_data_on_disable',
            __('Remove all data', 'da-reactions'),
            array($this, 'renderRemoveDataOnDisableCheckbox'),
            $this->options_page,
            $section
        );
    }
    /**
     * Register User Settings Section
     *
     * @since 3.1.1
     */
    public function registerUserSettings()
    {
        $section = 'user_section';
        $title = __('User settings', 'da-reactions');
        $intro = __('Users’ related settings.', 'da-reactions');
        add_settings_section(
            $section,
            $title,
            $this->makeSectionRenderer($intro),
            $this->options_page
        );
            add_settings_field(
                'da_r_user_restrictions',
                __('Restrict reactions usage for user roles', 'da-reactions'),
                $this->makeCheckboxRenderer(
                    'user_roles_restriction',
                    __('Enable restrictions.', 'da-reactions')
                ),
                $this->options_page,
                $section,
                array(
                    'class' => 'user_restrict_selector'
                )
            );
            add_settings_field(
                'da_r_user_types',
                __('Enable reactions for this kind of users', 'da-reactions'),
                array($this, 'renderUserRoleSelector__premium_only'),
                $this->options_page,
                $section,
                array(
                    'class' => 'user_type_selector'
                )
            );
        add_settings_field(
            'da_r_user_can_change',
            __('Can users change their reactions?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'user_can_change_reaction',
                __('User can change reaction.', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
        add_settings_field(
            'da_r_user_can_remove',
            __('Can users remove their reactions?', 'da-reactions'),
            $this->makeCheckboxRenderer(
                'user_can_remove_reaction',
                __('User can remove own reaction.', 'da-reactions')
            ),
            $this->options_page,
            $section
        );
    }
    /**
     * Render function for color type selector
     *
     * @since 1.3.2
     */
    public function renderChartColorSelector()
    {
        $field_name = $this->options->getFieldName("chart_colors");
        $saved_value = $this->options->getOption("chart_colors");
        ?>
        <p>
            <?php _e('Sometimes the icon colors are too similar to be used for graphics as well, so you can change the colors here.', 'da-reactions'); ?>
        </p>
        <p>
            <label for="id_<?= $field_name; ?>"><?php _e('Chart color scheme: ', 'da-reactions') ?></label>
            <select id="id_<?= $field_name; ?>" name="<?= $field_name; ?>">
                <option value="icons" <?= (empty($saved_value) || $saved_value === 'icons') ? 'selected = "selected"' : ''; ?>>
                    <?php _e('Use buttons’ colors', 'da-reactions'); ?></option>
                <option value="random" <?= ($saved_value === 'random') ? 'selected = "selected"' : ''; ?>>
                    <?php _e('Use random generated colors', 'da-reactions'); ?></option>
                <option value="default" <?= ($saved_value === 'default') ? 'selected = "selected"' : ''; ?>>
                    <?php _e('Use default rainbow palette', 'da-reactions'); ?></option>
            </select>
        </p>
        <?php
    }
    /**
     * Render function for page type checkgroup
     *
     * @since   1.0.0
     */
    public function renderPageTypeSelector()
    {
        $registered_page_types = json_decode('{
			"single": {
				"name": "single",
                "label": "' . __('Single pages and posts', 'da-reactions') . '"
            },
            "archive": {
				"name": "archive",
                "label": "' . __('Archives', 'da-reactions') . '"
            },
            "blog": {
				"name": "blog",
                "label": "' . __('Blog page', 'da-reactions') . '"
            }
        }');
        foreach ($registered_page_types as $page_type) {
            $field_name = $this->options->getFieldName("page_type_$page_type->name");
            $saved_value = $this->options->getOption("page_type_$page_type->name");
            $checked = isset($saved_value) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked($checked, 1) ?> value="on"/>
                <label for="id_<?= $field_name ?>"><?= $page_type->label ?></label>
            </p>
            <?php
        }
    }
    /**
     * Render function for post type checkgroup
     *
     * @since   1.0.0
     */
    public function renderPostTypeSelector()
    {
        $post_type_query = array(
            'public' => true,
        );
        $registered_post_types = get_post_types(
            $post_type_query,
            'objects'
        );
        foreach ($registered_post_types as $post_type) {
            if (isset($post_type->source) and $post_type->source === 'bbpress') {
                continue;
            }
            $field_name = $this->options->getFieldName("post_type_$post_type->name");
            $saved_value = $this->options->getOption("post_type_$post_type->name");
            $checked = isset($saved_value) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked($checked, 1) ?> value="on"/>
                <label for="id_<?= $field_name ?>"><?= $post_type->label ?></label>
                <?php
                if (post_type_supports($post_type->name, 'comments')) {
                    $post_type_name = $post_type->name;
                    $field_name = $this->options->getFieldName("post_type_${post_type_name}_comments");
                    $saved_value = $this->options->getOption("post_type_${post_type_name}_comments");
                    $checked = isset($saved_value) && $saved_value === 'on';
                    ?>
                    <input type="hidden" name="<?= $field_name ?>" value="off"/>
                    <input id="id_<?= $field_name ?>" type="checkbox"
                           name="<?= $field_name ?>" <?php checked($checked, 1) ?> value="on"/>
                    <label for="id_<?= $field_name ?>"><?php _e(' and their comments.', 'da-reactions'); ?></label>
                    <?php
                }
                ?>
            </p>
            <?php
        }
    }
    /**
     * Render function for remove data on plugin disable checkbox
     *
     * @since   1.0.0
     */
    public
    function renderRemoveDataOnDisableCheckbox()
    {
        if (!is_multisite() || is_main_site()) {
            $saved_value = $this->options->getOption('remove_data_on_disable');
            ?>
            <p>
                <a href="javascript:;" id="delete-all-data"
                   class="button button-link-delete"><?php _e('Delete', 'da-reactions'); ?></a>
                <?php _e('Delete all data. Warning! Lost data cannot be recovered. ', 'da-reactions'); ?>
            </p>
            <?php
        } else {
            $field_name = 'disabled_option';
            $main_site_options = Options::getInstance('general', get_main_site_id());
            $saved_value = $main_site_options->getOption('remove_data_on_disable');
            $checked = isset($saved_value) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="disabled" <?php checked($checked, 1) ?> value="on" disabled/>
                <label for="id_<?= $field_name ?>">
                    <?php _e('This option is only available in main blog settings because may affect all blogs in this network . ', 'da-reactions'); ?></label>
            </p>
            <?php
        }
    }
    /**
     * Render function for user identifying method checkgroup
     *
     * @since 1.0.0
     */
    public function renderUserIdentificationMethod()
    {
        $id_methods = array(
            'cookie' => array(
                'name' => __('Set cookie', 'da-reactions')
            ),
            'ip' => array(
                'name' => __('Save IP address', 'da-reactions')
            )
        );
        foreach ($id_methods as $method_slug => $method_info) {
            $field_name = $this->options->getFieldName("id_method_$method_slug");
            $saved_value = $this->options->getOption("id_method_$method_slug");
            $checked = isset($saved_value) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked($checked, 1) ?> value="on"/>
                <label for="id_<?= $field_name ?>"><?= $method_info['name'] ?></label>
            </p>
            <?php
        }
    }
    /**
     * Render function for user roles selection checkgroup
     *
     * @since 1.0.0
     */
    public function renderUserRoleSelector__premium_only()
    {
        $registered_user_roles = get_editable_roles();
        if (function_exists('bbp_get_dynamic_roles')) {
            $forum_user_roles = bbp_get_dynamic_roles();
            foreach ($forum_user_roles as $slug => $role) {
                if (!isset($registered_user_roles[$slug])) {
                    $registered_user_roles[$slug] = array(
                        'name' => "<strong>BBPress</strong> " . bbp_translate_user_role($role['name']),
                        'capabilities' => $role->capabilities
                    );
                }
            }
        }
        foreach ($registered_user_roles as $role_slug => $role_info) {
            $field_name = $this->options->getFieldName("user_role_$role_slug");
            $saved_value = $this->options->getOption("user_role_$role_slug");
            $checked = isset($saved_value) && $saved_value === 'on';
            ?>
            <p>
                <input type="hidden" name="<?= $field_name ?>" value="off"/>
                <input id="id_<?= $field_name ?>" type="checkbox"
                       name="<?= $field_name ?>" <?php checked($checked, 1) ?> value="on"/>
                <label for="id_<?= $field_name ?>"><?= $role_info['name'] ?></label>
            </p>
            <?php
        }
        //// Add unregistered users
        $role_slug = 'unregistered';
        $role_info = array(
            'name' => __('Unregistered', 'da-reactions')
        );
        $field_name = $this->options->getFieldName("user_role_$role_slug");
        $saved_value = $this->options->getOption("user_role_$role_slug");
        $checked = isset($saved_value) && $saved_value === 'on';
        ?>
        <p>
            <input type="hidden" name="<?= $field_name ?>" value="off"/>
            <input id="id_<?= $field_name ?>" type="checkbox"
                   name="<?= $field_name ?>" <?php checked($checked, 1) ?>
                   value="on"/>
            <label for="id_<?= $field_name ?>"><?= $role_info['name'] ?></label>
        </p>
        <?php
    }
    /**
     * Should validate input data, do nothing for now
     *
     * @param array $input
     *
     * @return array
     *
     * @since 1.0.0
     */
    public function sanitizeData(
        $input
    )
    {
        /**
         * Preserve previously saved data
         *
         * @since 3.1.1
         */
        $saved_options = $this->options->getAllOptions();
        foreach ($input as $key => $value) {
            $saved_options[$key] = $value;
        }
        return $saved_options;
    }
}
